/* 
 * File:   main.c
 * Author: th
 * Display routine for Display with HD77480 controller
 * Bally 35 display version
 * for 4 line 20 chars LED diplay
 * PIC 18F45K22
 * READ Port / Write LAT
 */

//no warnings about 
#pragma warning push
#pragma warning disable 359 //illegal conversion between pointer types
#pragma warning disable 373 //'implicite signed to unsigned conversion'

#include "def.h"
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <pic18f45k22.h>
#include "lcd_hd44780.h"

#define VERSION "0.1"

//typedefs
typedef union typed {
    uint8_t byte;
    struct {    
    unsigned b0:1, b1:1, b2:1, b3:1, b4:1, b5:1, b6:1, b7:1;
        } bitv;
    struct {    
    // this is segment order for HW Version 2.12   RB0-RB7 aceghfdb
    //unsigned a:1, c:1, e:1, g:1, h:1, f:1, d:1, b:1;
    unsigned b:1, d:1, f:1, h:1, g:1, e:1, c:1, a:1;    
        } segv;        
    } bitv_t;
    
//typedefs
typedef union typed2 {
    uint8_t byte;
    struct {    
    unsigned b0:1, b1:1, b2:1, b3:1, b4:1, b5:1, b6:1, b7:1;
        } bitv;
    struct {    
    // the strobes
    unsigned b0:1, b1:1, b2:1, b3:1, strobe_4:1, strobe_5:1, strobe_6:1, strobe_7:1;    
        } strv;        
    } strobe_t;
    
/* global definitions */  
volatile uint8_t Data_Display[7];


/* interrut service routine */
void __interrupt(low_priority) low_prio_Isr(void) {        
    
    strobe_t my_PORTB;
    static strobe_t my_old_PORTB;
    
  // interrupt on change?
  if (INTCONbits.RBIF ) //At least one of the IOC<3:0> (RB<7:4>) pins changed state
  {      
      // read of the port, as per datasheet
      my_PORTB.byte = PORTB;
        
      //do we have a change from 0 to 1 ?
      //if yes, assign new display data to buffer
      //we assign data in main
      if( ( my_PORTB.strv.strobe_4 == 1 ) & ( my_old_PORTB.strv.strobe_4 == 0 ))
      {
          Data_Display[3] = PORTA & 0x0f; 
      }
      else if( ( my_PORTB.strv.strobe_5 == 1 ) & ( my_old_PORTB.strv.strobe_5 == 0 ))
      {
          Data_Display[2] = PORTA & 0x0f; 
      }
      else if( ( my_PORTB.strv.strobe_6 == 1 ) & ( my_old_PORTB.strv.strobe_6 == 0 ))
      {
          Data_Display[1] = PORTA & 0x0f; 
      }
      else if( ( my_PORTB.strv.strobe_7 == 1 ) & ( my_old_PORTB.strv.strobe_7 == 0 ))
      {
          Data_Display[0] = PORTA & 0x0f; 
      }
      
    //store old value
    my_old_PORTB.byte = my_PORTB.byte;
    
    INTCONbits.RBIF = 0; // clear this interrupt condition
  }     
    
  // external INT2 ??
  if (INTCON3bits.INT2IF ) 
  {      
    //RB2/INT2 is status display
    Data_Display[4] = PORTA & 0x0f; 
    
    INTCON3bits.INT2IF = 0; // clear this interrupt condition
  }
}

/*
 * get the char for a Bally BCD value
 */
char get_char( uint8_t bcd_val)
{
    char ret_char;
    
    switch(bcd_val)
    {
        case 0 :   ret_char = '0'; break;                        
        case 1 :   ret_char = '1'; break;
        case 2 :   ret_char = '2'; break;                        
        case 3 :   ret_char = '3'; break;
        case 4 :   ret_char = '4'; break;
        case 5 :   ret_char = '5'; break;
        case 6 :   ret_char = '6'; break;
        case 7 :   ret_char = '7'; break;
        case 8 :   ret_char = '8'; break;
        case 9 :   ret_char = '9'; break;
        case 15 :   ret_char = ' '; break;
        default: ret_char = '?';
    }
    
return(ret_char);
    
}
/*
 * Main
 */
void main(void) {
    
    int i;
    char line1[21], line2[21], line3[21], line4[21];
    char oldline1[21], oldline2[21], oldline3[21], oldline4[21];
    uint8_t blank,old_blank,digit;
    

uint8_t status[7];     
uint8_t player1[7];
uint8_t player2[7];
uint8_t player3[7];
uint8_t player4[7];
uint8_t player5[7]; 
uint8_t player6[7];


    
//Clock Source and Value: Internal 16MHz
OSCCONbits.IRCF0 = 1; 
OSCCONbits.IRCF1 = 1;
OSCCONbits.IRCF2 = 1;
//Device is running from the internal oscillator
OSCCONbits.OSTS = 0;
//Primary clock (determined by FOSC<3:0> in CONFIG1H).
OSCCONbits.SCS = 0;
//OSCCONbits.SCS = 3; //RTH: 64MHz still need to be tested
// turn off the PLL 16*4
OSCTUNEbits.PLLEN = 0;      
//OSCTUNEbits.PLLEN = 1;      // turn on the PLL 16*4

// Wait until PLL is ready
//while(!OSCCON2bits.PLLRDY);

/* Port init */
// 0 is output; 1 is input
// Port A
ANSELA = 0;   //All I/O pins are configured as digital
LATA = 0;     //clear port
TRISA = 0b10111111;    //RA6 output (R/W))

//---------------------------------------------
// Port B
ANSELB = 0;   //All I/O pins are configured as digital
//TRISB = 0b11111110;    // RB0 output (RS))

//---------------------------------------------
//Port C
ANSELC = 0;   //All I/O pins are configured as digital
TRISC = 0b11100000; //RC0-RC4 output (LED E & data 4..7)

//---------------------------------------------
// Port D
ANSELD = 0;   //All I/O pins are configured as digital
TRISD = 0b11111110;    // D0 output ( RS )

//---------------------------------------------
// Port E
ANSELE = 0;   //All I/O pins are configured as digital
TRISE = 0b1111;    // all input

//INT2
INTCON2bits.INTEDG2 = 1; //1 = Interrupt on rising edge
INTCON3bits.INT2IE = 1; //1 = Enables the INT2 external interrupt


//interrupt on change RB 7-4
INTCONbits.RBIE = 1;  //Enables the IOCx port change interrupt
IOCBbits.IOCB4 =1;
IOCBbits.IOCB5 =1;
IOCBbits.IOCB6 =1;
IOCBbits.IOCB7 =1;

INTCONbits.GIE_GIEH  = 1;       // GIE/GIEH: Global Interrupt Enable bit
INTCONbits.PEIE_GIEL = 1;       // PEIE/GIEL: Peripheral Interrupt Enable bit

/* init */
LCD_INIT();
for(i=0; i<=6; i++) player1[i] = player2[i] = player3[i] = player4[i] = ' ';
for(i=0; i<=5; i++) player5[i] = player6[i] = ' ';
for(i=0; i<=3; i++) status[i] = ' ';
strcpy( oldline1,"");strcpy( oldline2,"");strcpy( oldline3,"");strcpy( oldline4,"");

LCD_CMD(LCD_CMD_CLR); // Display loeschen

//initial output
sprintf(line1,"BADI80 Version %s",VERSION);
LCD_GotoXY(0,0);
LCD_STRING(line1);
LCD_GotoXY(0,1);
LCD_STRING("   www.lisy.dev");
LCD_GotoXY(0,2);
LCD_STRING("    READY");
LCD_GotoXY(0,3);
LCD_STRING(" wait for Bally    ");

//wait a second
for(i=0; i<=100; i++) __delay_ms(10);

// Display loeschen
LCD_CMD(LCD_CMD_CLR); 

 while(1)
  { 
     //no INT IOs free, so we check Blanking in a loop
     blank = BLANKING;
     
     //did we got a 1->0 transition?
     if ( ( blank == 0) & ( old_blank == 1))
     {
         //for which digit?
         digit = 0; //default
         if (DIGIT_1) digit = 0;
         else if (DIGIT_2) digit = 1;
         else if (DIGIT_3) digit = 2;
         else if (DIGIT_4) digit = 3;
         else if (DIGIT_5) digit = 4;
         else if (DIGIT_6) digit = 5;
         //else if (DIGIT_7) digit = 6;
         
         //assign         
         player1[digit] = Data_Display[0];
         player2[digit] = Data_Display[1];
         player3[digit] = Data_Display[2];
         player4[digit] = Data_Display[3];
         status[digit] = Data_Display[4];
         player5[digit] = Data_Display[5];
         player6[digit] = Data_Display[6];
                  
         //assign player value to lines
         //6digit onl at the moment
             //line 1 is player1 & player2          
     line1[0] = line1[1] = line1[2] = ' ';
     line1[3] = get_char(player1[5]);
     line1[4] = get_char(player1[4]);
     line1[5] = get_char(player1[3]);
     line1[6] = get_char(player1[2]);
     line1[7] = get_char(player1[1]);
     line1[8] = get_char(player1[0]);
     //line1[9] = get_char(player1[6]);
     line1[9] = line1[10] = line1[11] = line1[12] = ' ';
     line1[13] = get_char(player2[5]);
     line1[14] = get_char(player2[4]);
     line1[15] = get_char(player2[3]);
     line1[16] = get_char(player2[2]);
     line1[17] = get_char(player2[1]);
     line1[18] = get_char(player2[0]);
     //line1[19] = get_char(player2[6]);
     line1[19] = '\0';
    //line 2 is player3 & player4          
     line2[0] = line2[1] = line2[2] = ' ';
     line2[3] = get_char(player3[5]);
     line2[4] = get_char(player3[4]);
     line2[5] = get_char(player3[3]);
     line2[6] = get_char(player3[2]);
     line2[7] = get_char(player3[1]);
     line2[8] = get_char(player3[0]);
     //line2[9] = get_char(player3[6]);
     line2[9] = line2[10] = line2[11] = line2[12] = ' ';
     line2[13] = get_char(player4[5]);
     line2[14] = get_char(player4[4]);
     line2[15] = get_char(player4[3]);
     line2[16] = get_char(player4[2]);
     line2[17] = get_char(player4[1]);
     line2[18] = get_char(player4[0]);
     //line2[19] = get_char(player4[6]);
     line2[19] = '\0';     
     //line 3 is display 5&6 TBD
     /*
     line3[0] = line3[1] = line3[2] = ' ';
     line3[3] = get_char(player5[5]);
     line3[4] = get_char(player5[4]);
     line3[5] = get_char(player5[3]);
     line3[6] = get_char(player5[2]);
     line3[7] = get_char(player5[1]);
     line3[8] = get_char(player5[0]);
     line3[9] = ' ';
     line3[10] = line3[11] = line3[12] = ' ';
     line3[13] = get_char(player6[5]);
     line3[14] = get_char(player6[4]);
     line3[15] = get_char(player6[3]);
     line3[16] = get_char(player6[2]);
     line3[17] = get_char(player6[1]);
     line3[18] = get_char(player6[0]);
     line3[19] = ' ';
     line3[20] = '\0';
      * */
    //line 4 ist status
     line4[0] = line4[1] = ' ';
     line4[2] = get_char(status[5]);
     line4[3] = get_char(status[4]);
     line4[4] = get_char(status[3]);
     line4[5] = get_char(status[2]);     
     line4[6] = get_char(status[1]);
     line4[7] = get_char(status[0]);
     line4[8] = '\0';     
         
         //something changed?
         //show it
     if ( strcmp(line1,oldline1) != 0)
     {      
        strcpy(oldline1,line1);
        LCD_GotoXY(0,0);
        LCD_STRING(line1);
     }
     if ( strcmp(line2,oldline2) != 0)
     {           
        strcpy(oldline2,line2);
        LCD_GotoXY(0,1);        
        LCD_STRING(line2);     
     }
     /*
     if ( strcmp(line3,oldline3) != 0)
     {  
        strcpy(oldline3,line3); 
        LCD_GotoXY(0,2);                  
        LCD_STRING(line3);
     }
      * */
     if ( strcmp(line4,oldline4) != 0)
     {      
        strcpy(oldline4,line4);
        LCD_GotoXY(0,3);        
        LCD_STRING(line4);                    
     }
     }
     
         //mem old value
         old_blank = blank;

  }//endless loop

}//main
   

